/***********************************************************************************************
 * File Info: $Id: NonStackedDataProcessor.java,v 1.3 2003/02/17 20:02:56 nathaniel_auvil Exp $
 * Copyright (C) 2000
 * Author: Nathaniel G. Auvil
 * Contributor(s):
 *
 * Copyright 2002 (C) Nathaniel G. Auvil. All Rights Reserved.
 *
 * Redistribution and use of this software and associated documentation
 * ("Software"), with or without modification, are permitted provided
 * that the following conditions are met:
 *
 * 1. Redistributions of source code must retain copyright
 *    statements and notices.  Redistributions must also contain a
 *    copy of this document.
 *
 * 2. Redistributions in binary form must reproduce the
 *    above copyright notice, this list of conditions and the
 *    following disclaimer in the documentation and/or other
 *    materials provided with the distribution.
 *
 * 3. The name "jCharts" or "Nathaniel G. Auvil" must not be used to
 * 	  endorse or promote products derived from this Software without
 * 	  prior written permission of Nathaniel G. Auvil.  For written
 *    permission, please contact nathaniel_auvil@users.sourceforge.net
 *
 * 4. Products derived from this Software may not be called "jCharts"
 *    nor may "jCharts" appear in their names without prior written
 *    permission of Nathaniel G. Auvil. jCharts is a registered
 *    trademark of Nathaniel G. Auvil.
 *
 * 5. Due credit should be given to the jCharts Project
 *    (http://jcharts.sourceforge.net/).
 *
 * THIS SOFTWARE IS PROVIDED BY Nathaniel G. Auvil AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL
 * jCharts OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 ************************************************************************************************/

package org.jCharts.chartData.processors;


import org.jCharts.chartData.interfaces.IAxisChartDataSet;


/*******************************************************************************************
 *
 *
 *******************************************************************************************/
abstract class NonStackedDataProcessor
{

	/*******************************************************************************************
	 * Does a single pass through the data set and calculates the  min, max, and label dimensions.
	 *  This stuff is grouped together so we can try to limit the number of passes through the data.
	 *
	 * @param iAxisChartDataSet
	 * @param axisChartDataProcessor
	 ********************************************************************************************/
	public static final void processData( IAxisChartDataSet iAxisChartDataSet,
													  AxisChartDataProcessor axisChartDataProcessor )
	{
		//LOOP
		for( int i = 0; i < iAxisChartDataSet.getNumberOfDataItems(); i++ )
		{
			//LOOP
			for( int j = 0; j < iAxisChartDataSet.getNumberOfDataSets(); j++ )
			{
				if( Double.isNaN( iAxisChartDataSet.getValue( j, i ) ) )
				{
					continue;
				}

				if( iAxisChartDataSet.getValue( j, i ) > axisChartDataProcessor.getMaxValue() )
				{
					axisChartDataProcessor.setMaxValue( iAxisChartDataSet.getValue( j, i ) );
				}

				if( iAxisChartDataSet.getValue( j, i ) < axisChartDataProcessor.getMinValue() )
				{
					axisChartDataProcessor.setMinValue( iAxisChartDataSet.getValue( j, i ) );
				}
			}
		}
	}
}
